 MODULE diffusion_u_mod

!------------------------------------------------------------------------------!
! Actual revisions:
! -----------------
! 
! 
! Former revisions:
! -----------------
! $Log: diffusion_u.f90,v $
! Revision 1.15  2006/02/23 10:35:35  raasch
! nzb_2d replaced by nzb_u_outer in horizontal diffusion and by nzb_u_inner
! or nzb_diff_u, respectively, in vertical diffusion,
! wall functions added for north and south walls, +z0 in argument list,
! terms containing w(k-1,..) are removed from the Prandtl-layer equation
! because they cause errors at the edges of topography
! WARNING: loops containing the MAX function are still not properly vectorized!
!
! Revision 1.14  2005/03/26 20:08:21  raasch
! Extension of horizontal loop upper bounds for non-cyclic boundary conditions,
! additional damping layer at the outflow in case of non-cyclic lateral
! boundaries, additional argument km_damp_y
!
! Revision 1.13  2004/01/30 10:21:32  raasch
! Scalar lower k index nzb replaced by 2d-array nzb_2d
!
! Revision 1.12  2003/03/12 16:26:14  raasch
! Full code replaced in the call for all gridpoints instead of calling the
! _ij version (required by NEC, because otherwise no vectorization)
!
! Revision 1.11  2002/06/11 12:53:12  raasch
! Former subroutine changed to a module which allows to be called for all grid
! points of a single vertical column with index i,j or for all grid points by
! using function overloading.
!
! Revision 1.10  2001/08/21 08:25:55  raasch
! Special treatment at k=1 generally if momentum flux is prescribed (not only in
! case of a Prandtl layer)
!
! Revision 1.9  2001/03/30 07:12:02  raasch
! Translation of remaining German identifiers (variables, subroutines, etc.)
!
! Revision 1.8  2001/01/22 06:14:00  raasch
! Module test_variables removed
!
! Revision 1.7  2000/07/03 12:57:46  raasch
! dummy arguments, whose corresponding actual arguments are pointers,
! are now also defined as pointers,
! all comments translated into English
!
! Revision 1.6  2000/03/14 13:55:33  schroeter
! vertical impulse fluxes are computed over dz at the
! top of the Prandtl-layer
!
! Revision 1.5  98/07/06  12:11:42  12:11:42  raasch (Siegfried Raasch)
! + USE test_variables
! 
! Revision 1.4  1998/01/23 09:56:44  raasch
! Einbau der Prandtl-Schicht
!
! Revision 1.3  1997/09/12 07:22:50  raasch
! Leerzeilen mussten entfernt werden
!
! Revision 1.2  1997/09/12 06:43:27  raasch
! HP-Compiler erfordert & am Beginn von Fortsetzungszeilen
!
! Revision 1.1  1997/09/12 06:23:51  raasch
! Initial revision
!
!
! Description:
! ------------
! Diffusion term of the u-component
!------------------------------------------------------------------------------!

    PRIVATE
    PUBLIC diffusion_u

    INTERFACE diffusion_u
       MODULE PROCEDURE diffusion_u
       MODULE PROCEDURE diffusion_u_ij
    END INTERFACE diffusion_u

 CONTAINS


!------------------------------------------------------------------------------!
! Call for all grid points
!------------------------------------------------------------------------------!
    SUBROUTINE diffusion_u( ddzu, ddzw, km, km_damp_y, tend, u, usws, v, w, z0 )

       USE control_parameters
       USE grid_variables
       USE indices

       IMPLICIT NONE

       INTEGER ::  i, j, k
       REAL    ::  kmym_x, kmym_y, kmyp_x, kmyp_y, kmzm, kmzp, usvs
       REAL    ::  ddzu(1:nzt+1), ddzw(1:nzt), km_damp_y(nys-1:nyn+1)
       REAL    ::  z0(nys-1:nyn+1,nxl-1:nxr+1)
       REAL    ::  tend(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1)
       REAL, DIMENSION(:,:),   POINTER ::  usws
       REAL, DIMENSION(:,:,:), POINTER ::  km, u, v, w

       DO  i = nxl, nxr+uxrp
          DO  j = nys,nyn
!
!--          Compute horizontal diffusion
             DO  k = nzb_u_outer(j,i)+1, nzt
!
!--             Interpolate eddy diffusivities on staggered gridpoints
                kmyp_x = 0.25 * &
                         ( km(k,j,i)+km(k,j+1,i)+km(k,j,i-1)+km(k,j+1,i-1) )
                kmym_x = 0.25 * &
                         ( km(k,j,i)+km(k,j-1,i)+km(k,j,i-1)+km(k,j-1,i-1) )
                kmyp_y = kmyp_x
                kmym_y = kmym_x
!
!--             Increase diffusion at the outflow boundary in case of
!--             non-cyclic lateral boundaries. Damping is only needed for
!--             velocity components parallel to the outflow boundary in
!--             the direction normal to the outflow boundary.
                IF ( bc_ns /= 'cyclic' )  THEN
                   kmyp_y = MAX( kmyp_y, km_damp_y(j) )
                   kmym_y = MAX( kmym_y, km_damp_y(j) )
                ENDIF

                tend(k,j,i) = tend(k,j,i)                                    &
                      & + 2.0 * (                                            &
                      &           km(k,j,i)   * ( u(k,j,i+1) - u(k,j,i)   )  &
                      &         - km(k,j,i-1) * ( u(k,j,i)   - u(k,j,i-1) )  &
                      &         ) * ddx2                                     &
                      & + ( kmyp_y * ( u(k,j+1,i) - u(k,j,i)     ) * ddy     &
                      &   + kmyp_x * ( v(k,j+1,i) - v(k,j+1,i-1) ) * ddx     &
                      &   - kmym_y * ( u(k,j,i) - u(k,j-1,i) ) * ddy         &
                      &   - kmym_x * ( v(k,j,i) - v(k,j,i-1) ) * ddx         &
                      &   ) * ddy
             ENDDO

!
!--          Wall functions at the north and south walls, respectively
             IF ( wall_u(j,i) /= 0.0 )  THEN
                DO  k = nzb_u_inner(j,i)+1, nzb_u_outer(j,i)
                   usvs   = kappa * u(k,j,i) / LOG( 0.5 * dy / z0(j,i) )
                   usvs   = -usvs * ABS( usvs )
                   kmyp_x = 0.25 * &
                            ( km(k,j,i)+km(k,j+1,i)+km(k,j,i-1)+km(k,j+1,i-1) )
                   kmym_x = 0.25 * &
                            ( km(k,j,i)+km(k,j-1,i)+km(k,j,i-1)+km(k,j-1,i-1) )
                   kmyp_y = kmyp_x
                   kmym_y = kmym_x
!
!--                Increase diffusion at the outflow boundary in case of
!--                non-cyclic lateral boundaries. Damping is only needed for
!--                velocity components parallel to the outflow boundary in
!--                the direction normal to the outflow boundary.
                   IF ( bc_ns /= 'cyclic' )  THEN
                      kmyp_y = MAX( kmyp_y, km_damp_y(j) )
                      kmym_y = MAX( kmym_y, km_damp_y(j) )
                   ENDIF

                   tend(k,j,i) = tend(k,j,i)                                   &
                                 + 2.0 * (                                     &
                                       km(k,j,i)   * ( u(k,j,i+1) - u(k,j,i) ) &
                                     - km(k,j,i-1) * ( u(k,j,i) - u(k,j,i-1) ) &
                                         ) * ddx2                              &
                                 + (   fyp(j,i) * (                            &
                                  kmyp_y * ( u(k,j+1,i) - u(k,j,i)     ) * ddy &
                                + kmyp_x * ( v(k,j+1,i) - v(k,j+1,i-1) ) * ddx &
                                                  )                            &
                                     - fym(j,i) * (                            &
                                  kmym_y * ( u(k,j,i) - u(k,j-1,i) ) * ddy     &
                                + kmym_x * ( v(k,j,i) - v(k,j,i-1) ) * ddx     &
                                                  )                            &
                                     + wall_u(j,i) * usvs                      &
                                   ) * ddy
                ENDDO
             ENDIF

!
!--          Compute vertical diffusion. In case of simulating a Prandtl layer,
!--          index k starts at nzb_u_inner+2.
             DO  k = nzb_diff_u(j,i), nzt
!
!--             Interpolate eddy diffusivities on staggered gridpoints
                kmzp = 0.25 * &
                       ( km(k,j,i)+km(k+1,j,i)+km(k,j,i-1)+km(k+1,j,i-1) )
                kmzm = 0.25 * &
                       ( km(k,j,i)+km(k-1,j,i)+km(k,j,i-1)+km(k-1,j,i-1) )

                tend(k,j,i) = tend(k,j,i)                                    &
                      & + ( kmzp * ( ( u(k+1,j,i) - u(k,j,i)   ) * ddzu(k+1) &
                      &            + ( w(k,j,i)   - w(k,j,i-1) ) * ddx       &
                      &            )                                         &
                      &   - kmzm * ( ( u(k,j,i)   - u(k-1,j,i)   ) * ddzu(k) &
                      &            + ( w(k-1,j,i) - w(k-1,j,i-1) ) * ddx     &
                      &            )                                         &
                      &   ) * ddzw(k)
             ENDDO

!
!--          Vertical diffusion at the first grid point above the surface,
!--          if the momentum flux at the bottom is given by the Prandtl law or
!--          if it is prescribed by the user.
!--          Difference quotient of the momentum flux is not formed over half
!--          of the grid spacing (2.0*ddzw(k)) any more, since the comparison
!--          with other (LES) modell showed that the values of the momentum
!--          flux becomes too large in this case.
!--          The term containing w(k-1,..) (see above equation) is removed here
!--          because the vertical velocity is assumed to be zero at the surface.
             IF ( use_surface_fluxes )  THEN
                k = nzb_u_inner(j,i)+1
!
!--             Interpolate eddy diffusivities on staggered gridpoints
                kmzp = 0.25 * &
                      ( km(k,j,i)+km(k+1,j,i)+km(k,j,i-1)+km(k+1,j,i-1) )
                kmzm = 0.25 * &
                      ( km(k,j,i)+km(k-1,j,i)+km(k,j,i-1)+km(k-1,j,i-1) )

                tend(k,j,i) = tend(k,j,i)                                    &
                      & + ( kmzp * ( w(k,j,i)   - w(k,j,i-1)   ) * ddx       &
                      &   ) * ddzw(k)                                        &
                      & + ( kmzp * ( u(k+1,j,i) - u(k,j,i)   ) * ddzu(k+1)   &
                      &   + usws(j,i)                                        &
                      &   ) * ddzw(k)
             ENDIF

          ENDDO
       ENDDO

    END SUBROUTINE diffusion_u


!------------------------------------------------------------------------------!
! Call for grid point i,j
!------------------------------------------------------------------------------!
    SUBROUTINE diffusion_u_ij( i, j, ddzu, ddzw, km, km_damp_y, tend, u, usws, &
                               v, w, z0 )

       USE control_parameters
       USE grid_variables
       USE indices

       IMPLICIT NONE

       INTEGER ::  i, j, k
       REAL    ::  kmym_x, kmym_y, kmyp_x, kmyp_y, kmzm, kmzp, usvs
       REAL    ::  ddzu(1:nzt+1), ddzw(1:nzt), km_damp_y(nys-1:nyn+1)
       REAL    ::  z0(nys-1:nyn+1,nxl-1:nxr+1)
       REAL    ::  tend(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1)
       REAL, DIMENSION(:,:),   POINTER ::  usws
       REAL, DIMENSION(:,:,:), POINTER ::  km, u, v, w

!
!--    Compute horizontal diffusion
       DO  k = nzb_u_outer(j,i)+1, nzt
!
!--       Interpolate eddy diffusivities on staggered gridpoints
          kmyp_x = 0.25 * ( km(k,j,i)+km(k,j+1,i)+km(k,j,i-1)+km(k,j+1,i-1) )
          kmym_x = 0.25 * ( km(k,j,i)+km(k,j-1,i)+km(k,j,i-1)+km(k,j-1,i-1) )
          kmyp_y = kmyp_x
          kmym_y = kmym_x

!
!--       Increase diffusion at the outflow boundary in case of non-cyclic
!--       lateral boundaries. Damping is only needed for velocity components
!--       parallel to the outflow boundary in the direction normal to the
!--       outflow boundary.
          IF ( bc_ns /= 'cyclic' )  THEN
             kmyp_y = MAX( kmyp_y, km_damp_y(j) )
             kmym_y = MAX( kmym_y, km_damp_y(j) )
          ENDIF

          tend(k,j,i) = tend(k,j,i)                                          &
                      & + 2.0 * (                                            &
                      &           km(k,j,i)   * ( u(k,j,i+1) - u(k,j,i)   )  &
                      &         - km(k,j,i-1) * ( u(k,j,i)   - u(k,j,i-1) )  &
                      &         ) * ddx2                                     &
                      & + ( kmyp_y * ( u(k,j+1,i) - u(k,j,i)     ) * ddy     &
                      &   + kmyp_x * ( v(k,j+1,i) - v(k,j+1,i-1) ) * ddx     &
                      &   - kmym_y * ( u(k,j,i) - u(k,j-1,i) ) * ddy         &
                      &   - kmym_x * ( v(k,j,i) - v(k,j,i-1) ) * ddx         &
                      &   ) * ddy
       ENDDO

!
!--    Wall functions at the north and south walls, respectively
       IF ( wall_u(j,i) .NE. 0.0 )  THEN
          DO  k = nzb_u_inner(j,i)+1, nzb_u_outer(j,i)
             usvs   = kappa * u(k,j,i) / LOG( 0.5 * dy / z0(j,i) )
             usvs   = -usvs * ABS( usvs )
             kmyp_x = 0.25 * ( km(k,j,i)+km(k,j+1,i)+km(k,j,i-1)+km(k,j+1,i-1) )
             kmym_x = 0.25 * ( km(k,j,i)+km(k,j-1,i)+km(k,j,i-1)+km(k,j-1,i-1) )
             kmyp_y = kmyp_x
             kmym_y = kmym_x
!
!--          Increase diffusion at the outflow boundary in case of
!--          non-cyclic lateral boundaries. Damping is only needed for
!--          velocity components parallel to the outflow boundary in
!--          the direction normal to the outflow boundary.
             IF ( bc_ns /= 'cyclic' )  THEN
                kmyp_y = MAX( kmyp_y, km_damp_y(j) )
                kmym_y = MAX( kmym_y, km_damp_y(j) )
             ENDIF

             tend(k,j,i) = tend(k,j,i)                                         &
                                 + 2.0 * (                                     &
                                       km(k,j,i)   * ( u(k,j,i+1) - u(k,j,i) ) &
                                     - km(k,j,i-1) * ( u(k,j,i) - u(k,j,i-1) ) &
                                         ) * ddx2                              &
                                 + (   fyp(j,i) * (                            &
                                  kmyp_y * ( u(k,j+1,i) - u(k,j,i)     ) * ddy &
                                + kmyp_x * ( v(k,j+1,i) - v(k,j+1,i-1) ) * ddx &
                                                  )                            &
                                     - fym(j,i) * (                            &
                                  kmym_y * ( u(k,j,i) - u(k,j-1,i) ) * ddy     &
                                + kmym_x * ( v(k,j,i) - v(k,j,i-1) ) * ddx     &
                                                  )                            &
                                     + wall_u(j,i) * usvs                      &
                                   ) * ddy
          ENDDO
       ENDIF

!
!--    Compute vertical diffusion. In case of simulating a Prandtl layer,
!--    index k starts at nzb_u_inner+2.
       DO  k = nzb_diff_u(j,i), nzt
!
!--       Interpolate eddy diffusivities on staggered gridpoints
          kmzp = 0.25 * ( km(k,j,i)+km(k+1,j,i)+km(k,j,i-1)+km(k+1,j,i-1) )
          kmzm = 0.25 * ( km(k,j,i)+km(k-1,j,i)+km(k,j,i-1)+km(k-1,j,i-1) )

          tend(k,j,i) = tend(k,j,i)                                          &
                      & + ( kmzp * ( ( u(k+1,j,i) - u(k,j,i)   ) * ddzu(k+1) &
                      &            + ( w(k,j,i)   - w(k,j,i-1) ) * ddx       &
                      &            )                                         &
                      &   - kmzm * ( ( u(k,j,i)   - u(k-1,j,i)   ) * ddzu(k) &
                      &            + ( w(k-1,j,i) - w(k-1,j,i-1) ) * ddx     &
                      &            )                                         &
                      &   ) * ddzw(k)
       ENDDO

!
!--    Vertical diffusion at the first grid point above the surface, if the
!--    momentum flux at the bottom is given by the Prandtl law or if it is
!--    prescribed by the user.
!--    Difference quotient of the momentum flux is not formed over half of
!--    the grid spacing (2.0*ddzw(k)) any more, since the comparison with 
!--    other (LES) modell showed that the values of the momentum flux becomes
!--    too large in this case.
!--    The term containing w(k-1,..) (see above equation) is removed here
!--    because the vertical velocity is assumed to be zero at the surface.
       IF ( use_surface_fluxes )  THEN
          k = nzb_u_inner(j,i)+1
!
!--       Interpolate eddy diffusivities on staggered gridpoints
          kmzp = 0.25 * ( km(k,j,i)+km(k+1,j,i)+km(k,j,i-1)+km(k+1,j,i-1) )
          kmzm = 0.25 * ( km(k,j,i)+km(k-1,j,i)+km(k,j,i-1)+km(k-1,j,i-1) )

          tend(k,j,i) = tend(k,j,i)                                          &
                      & + ( kmzp * ( w(k,j,i)   - w(k,j,i-1)   ) * ddx       &
                      &   ) * ddzw(k)                                        &
                      & + ( kmzp * ( u(k+1,j,i) - u(k,j,i)   ) * ddzu(k+1)   &
                      &   + usws(j,i)                                        &
                      &   ) * ddzw(k)
       ENDIF

    END SUBROUTINE diffusion_u_ij

 END MODULE diffusion_u_mod
