 SUBROUTINE package_parin

!------------------------------------------------------------------------------!
! Actual revisions:
! -----------------
! 
!
! Former revisions:
! -----------------
! $Log: package_parin.f90,v $
! Revision 1.18  2006/08/04 14:52:23  raasch
! +dt_dopts, dt_min_part, end_time_prel, particles_per_point,
! use_sgs_for_particles in particles_par
!
! Revision 1.17  2006/04/11 14:59:05  raasch
! pl_spectra renamed data_output_sp
!
! Revision 1.16  2006/02/23 12:43:24  raasch
! +number_of_particle_groups, skip_particles_for_tail, use_particle_tails in
! particles_par,
! +skip_time_dosp in spectra_par,
! dt_plsp renamed dt_dosp, default setting of dt_dops,
! set flag particle_advection
!
! Revision 1.15  2005/06/26 20:05:42  raasch
! diameter renamed radius
!
! Revision 1.14  2003/10/29 09:01:29  raasch
! +diameter in particles_par, uniform_psize renamed dvrp_psize
!
! Revision 1.13  2003/08/01 08:43:33  raasch
! +slicer_range_limits_dvrp in dvrp_graphics_par
!
! Revision 1.12  2003/03/16 09:42:11  raasch
! Two underscores (_) are placed in front of all define-strings
!
! Revision 1.11  2002/09/12 13:06:50  raasch
! +density_ratio in particles_par
!
! Revision 1.10  2002/04/16 08:10:08  raasch
! +variables for particle boundary conditions, +vertical_particle_advection
! in particles_par
!
! Revision 1.9  2001/11/12 16:07:04  raasch
! +read_particles_from_restartfile
!
! Revision 1.7  2001/08/21 09:52:55  raasch
! +superelevation, superelevation_x, superelevation_y in dvrp_graphics_par,
! +maximum_number_of_particles, maximum_number_of_tailpoints,
! maximum_tailpoint_age, minimum_tailpoint_distance in particles_par
!
! Revision 1.6  2001/07/12 12:42:49  raasch
! +several new parameters in dvrp_graphics_par and particles_par
!
! Revision 1.5  2001/03/30 07:37:56  raasch
! Namelist names changed,
! Translation of remaining German identifiers (variables, subroutines, etc.)
!
! Revision 1.4  2001/01/25 07:10:48  raasch
! +write_particle_informations
!
! Revision 1.2  2001/01/05 15:13:34  raasch
! Spectra package added
!
! Revision 1.1  2000/12/28 13:21:57  raasch
! Initial revision
!
!
! Description:
! ------------
! This subroutine reads from the NAMELIST file variables controling model
! software packages which are used optionally in the run.
!------------------------------------------------------------------------------!

    USE control_parameters
    USE dvrp_variables
    USE particle_attributes
    USE spectrum

    IMPLICIT NONE

    CHARACTER (LEN=80) ::  zeile

    NAMELIST /dvrp_graphics_par/  dt_dvrp, dvrp_directory, dvrp_file,          &
                                  dvrp_host, dvrp_output, dvrp_password,       &
                                  dvrp_username, mode_dvrp,                    &
                                  slicer_range_limits_dvrp, superelevation,    &
                                  superelevation_x, superelevation_y, threshold
    NAMELIST /particles_par/      bc_par_b, bc_par_lr, bc_par_ns, bc_par_t,    &
                                  density_ratio, radius, dt_dopts,             &
                                  dt_min_part, dt_prel,                        &
                                  dt_write_particle_data, dvrp_psize,          &
                                  end_time_prel, initial_weighting_factor,     &
                                  maximum_number_of_particles,                 &
                                  maximum_number_of_tailpoints,                &
                                  maximum_tailpoint_age,                       &
                                  minimum_tailpoint_distance,                  &
                                  number_of_particle_groups,                   &
                                  particles_per_point,                         &
                                  particle_advection_start,                    &
                                  particle_maximum_age, pdx, pdy, pdz, psb,    &
                                  psl, psn, psr, pss, pst,                     &
                                  random_start_position,                       &
                                  read_particles_from_restartfile,             &
                                  skip_particles_for_tail, use_particle_tails, &
                                  use_sgs_for_particles,                       &
                                  vertical_particle_advection,                 &
                                  write_particle_statistics
    NAMELIST /spectra_par/        averaging_interval_sp, comp_spectra_level,   &
                                  data_output_sp, dt_dosp, plot_spectra_level, &
                                  skip_time_dosp, spectra_direction

!
!-- Position the namelist-file at the beginning (it was already opened in
!-- parin), search for the namelist-group of the package and position the
!-- file at this line. Do the same for each optionally used package.
    zeile = ' '

#if defined( __dvrp_graphics )
    REWIND ( 11 )
    zeile = ' '
    DO   WHILE ( INDEX( zeile, '&dvrp_graphics_par' ) == 0 )
       READ ( 11, '(A)', END=10 )  zeile
    ENDDO
    BACKSPACE ( 11 )

!
!-- Read user-defined namelist
    READ ( 11, dvrp_graphics_par )

 10 CONTINUE
#endif

#if defined( __particles )
    REWIND ( 11 )
    zeile = ' '
    DO   WHILE ( INDEX( zeile, '&particles_par' ) == 0 )
       READ ( 11, '(A)', END=20 )  zeile
    ENDDO
    BACKSPACE ( 11 )

!
!-- Read user-defined namelist
    READ ( 11, particles_par )

!
!-- Set flag that indicates that particles are switched on
    particle_advection = .TRUE.

 20 CONTINUE
#endif

#if defined( __spectra )
    REWIND ( 11 )
    zeile = ' '
    DO   WHILE ( INDEX( zeile, '&spectra_par' ) == 0 )
       READ ( 11, '(A)', END=30 )  zeile
    ENDDO
    BACKSPACE ( 11 )

!
!-- Read user-defined namelist
    READ ( 11, spectra_par )

!
!-- Default setting of dt_dosp here (instead of check_parameters), because its
!-- current value is needed in init_pegrid
    IF ( dt_dosp == 9999999.9 )  dt_dosp = dt_data_output

 30 CONTINUE
#endif

 END SUBROUTINE package_parin
