 SUBROUTINE write_3d_binary

!------------------------------------------------------------------------------!
! Actual revisions:
! -----------------
! 
!
! Former revisions:
! -----------------
! $Log: write_3d_binary.f90,v $
! Revision 1.21  2006/08/04 15:05:11  raasch
! +iran, iran_part
!
! Revision 1.20  2006/02/23 13:14:34  raasch
! +volume_flow_area, volume_flow_initial, -hom_sum_pr,
! output of arrays used for time-averaging
!
! Revision 1.19  2005/05/18 15:55:45  raasch
! pl1d_callcount renamed dopr_time_count
!
! Revision 1.18  2004/04/30 13:13:40  raasch
! Arrays of timelevel t-dt (_m) are written only in case of the leapfrog scheme
!
! Revision 1.17  2003/10/29 09:14:11  raasch
! Output of random seed (random_iv, random_iy), module random_function_mod
! is used, new binary version 2.2
!
! Revision 1.16  2002/12/19 16:23:08  raasch
! Writing of array hom moved to write_var_list
!
! Revision 1.15  2001/03/30 07:59:18  raasch
! Translation of remaining German identifiers (variables, subroutines, etc.),
! closing of unit 14 moved to the main program in order to allow user-defined
! writing to this unit
!
! Revision 1.14  2001/03/11 11:27:18  letzel
! All comments translated into English.
!
! Revision 1.13  2001/01/29 12:36:56  raasch
! Passive scalar is considered
!
! Revision 1.12  2001/01/22 08:26:59  raasch
! Module test_variables removed
!
! Revision 1.11  2001/01/05 15:16:39  raasch
! +spectrum_x, spectrum_y, new binary version 2.1
!
! Revision 1.10  2001/01/02 17:34:45  raasch
! Unit 14 closed at the end of the subroutine
!
! Revision 1.9  2000/04/13 14:12:37  schroeter
! Output of water and humidity variables
!
! Revision 1.8  99/03/03  09:27:43  09:27:43  raasch (Siegfried Raasch)
! Speziellen I/O-Messpunkt entfernt
! 
! Revision 1.7  1999/02/05 09:19:39  raasch
! program_version in binary_version umbenannt
!
! Revision 1.6  1998/12/15 07:34:40  raasch
! Zusaetzliche Ermittelung der tatsaechlichen Ausgabezeiten auf T3E
! mit messpunkt_e(15) unter Verwendung von TIMEF (s. cpu_zeitmessung)
!
! Revision 1.5  1998/07/06 12:36:26  raasch
! + USE test_variables
!
! Revision 1.4  1998/04/15 11:24:30  raasch
! Steuerparameter-Ausgaben gehen jetzt auch auf Einheit 14 und werden von
! jedem PE ausgegeben
!
! Revision 1.3  1998/04/06 14:34:21  raasch
! Ausgabe von weiteren Plotvariablen
!
! Revision 1.2  1998/03/30 11:39:00  raasch
! Statistik- und Plotvariablen werden zusaetzlich ausgegeben
!
! Revision 1.1  1998/03/18 20:20:21  raasch
! Initial revision
!
!
! Description:
! ------------
! Binary output of variables and arrays for restarts.
!------------------------------------------------------------------------------!

    USE arrays_3d
    USE averaging
    USE control_parameters
    USE cpulog
    USE indices
    USE interfaces
    USE particle_attributes
    USE pegrid
    USE profil_parameter
    USE random_function_mod
    USE statistics

    IMPLICIT NONE

    CHARACTER (LEN=10) ::  binary_version


    CALL cpu_log( log_point(22), 'write_3d_binary', 'start' )

    CALL check_open( 14 )
    
!
!-- Write control parameters and other variables for restart.
    CALL write_var_list

!
!-- Write arrays.
    binary_version = '3.0'

    WRITE ( 14 )  binary_version

    WRITE ( 14 )  numprocs, myid, nxl, nxr, nys, nyn, nzb, nzt

!
!-- Attention: After changes to the following output commands the version number
!-- ---------  of the variable binary_version must be changed!
!--            Also, the list of arrays to be read in read_3d_binary must be
!--            adjusted accordingly.

    WRITE ( 14 )  'e                   ';  WRITE ( 14 )  e
    IF ( ALLOCATED( e_av ) )  THEN
       WRITE ( 14 )  'e_av                ';  WRITE ( 14 )  e_av
    ENDIF
    WRITE ( 14 )  'e_m                 ';  WRITE ( 14 )  e_m
    WRITE ( 14 )  'iran                ';  WRITE ( 14 )  iran, iran_part
    WRITE ( 14 )  'kh                  ';  WRITE ( 14 )  kh
    IF ( timestep_scheme(1:5) /= 'runge' )  THEN
       WRITE ( 14 )  'kh_m                ';  WRITE ( 14 )  kh_m
    ENDIF
    WRITE ( 14 )  'km                  ';  WRITE ( 14 )  km
    IF ( timestep_scheme(1:5) /= 'runge' )  THEN
       WRITE ( 14 )  'km_m                ';  WRITE ( 14 )  km_m
    ENDIF
    IF ( ALLOCATED( lwp_av ) )  THEN
       WRITE ( 14 )  'lwp_av              ';  WRITE ( 14 )  lwp_av
    ENDIF
    WRITE ( 14 )  'p                   ';  WRITE ( 14 )  p
    IF ( ALLOCATED( p_av ) )  THEN
       WRITE ( 14 )  'p_av                ';  WRITE ( 14 )  p_av
    ENDIF
    IF ( ALLOCATED( pc_av ) )  THEN
       WRITE ( 14 )  'pc_av               ';  WRITE ( 14 )  pc_av
    ENDIF
    IF ( ALLOCATED( pr_av ) )  THEN
       WRITE ( 14 )  'pr_av               ';  WRITE ( 14 )  pr_av
    ENDIF
    WRITE ( 14 )  'pt                  ';  WRITE ( 14 )  pt
    IF ( ALLOCATED( pt_av ) )  THEN
       WRITE ( 14 )  'pt_av               ';  WRITE ( 14 )  pt_av
    ENDIF
    WRITE ( 14 )  'pt_m                ';  WRITE ( 14 )  pt_m
    IF ( moisture  .OR. passive_scalar )  THEN
       WRITE ( 14 )  'q                   ';  WRITE ( 14 )  q 
       IF ( ALLOCATED( q_av ) )  THEN
          WRITE ( 14 )  'q_av                ';  WRITE ( 14 )  q_av
       ENDIF
       WRITE ( 14 )  'q_m                 ';  WRITE ( 14 ) q_m
       IF ( cloud_physics ) THEN
          WRITE ( 14 )  'ql                  ';  WRITE ( 14 ) ql
          IF ( ALLOCATED( ql_av ) )  THEN
             WRITE ( 14 )  'ql_av               ';  WRITE ( 14 )  ql_av
          ENDIF
       ENDIF
       WRITE ( 14 )  'qs                  ';  WRITE ( 14 ) qs
       WRITE ( 14 )  'qsws                ';  WRITE ( 14 ) qsws
       IF ( timestep_scheme(1:5) /= 'runge' )  THEN
          WRITE ( 14 )  'qsws_m              ';  WRITE ( 14 ) qsws_m
       ENDIF
    ENDIF
    IF ( ALLOCATED( ql_c_av ) )  THEN
       WRITE ( 14 )  'ql_c_av             ';  WRITE ( 14 )  ql_c_av
    ENDIF
    IF ( ALLOCATED( ql_v_av ) )  THEN
       WRITE ( 14 )  'ql_v_av             ';  WRITE ( 14 )  ql_v_av
    ENDIF
    IF ( ALLOCATED( ql_vp_av ) )  THEN
       WRITE ( 14 )  'ql_vp_av            ';  WRITE ( 14 )  ql_vp_av
    ENDIF
    IF ( ALLOCATED( qv_av ) )  THEN
       WRITE ( 14 )  'qv_av               ';  WRITE ( 14 )  qv_av
    ENDIF
    WRITE ( 14 )  'random_iv           ';  WRITE ( 14 )  random_iv
                                           WRITE ( 14 )  random_iy
    WRITE ( 14 )  'rif                 ';  WRITE ( 14 )  rif
    IF ( timestep_scheme(1:5) /= 'runge' )  THEN
       WRITE ( 14 )  'rif_m               ';  WRITE ( 14 )  rif_m
    ENDIF
    IF ( ALLOCATED( s_av ) )  THEN
       WRITE ( 14 )  's_av                ';  WRITE ( 14 )  s_av
    ENDIF
    WRITE ( 14 )  'shf                 ';  WRITE ( 14 )  shf
    IF ( timestep_scheme(1:5) /= 'runge' )  THEN
       WRITE ( 14 )  'shf_m               ';  WRITE ( 14 )  shf_m
    ENDIF
    WRITE ( 14 )  'ts                  ';  WRITE ( 14 )  ts
    IF ( ALLOCATED( ts_av ) )  THEN
       WRITE ( 14 )  'ts_av               ';  WRITE ( 14 )  ts_av
    ENDIF
    WRITE ( 14 )  'u                   ';  WRITE ( 14 )  u
    IF ( ALLOCATED( u_av ) )  THEN
       WRITE ( 14 )  'u_av                ';  WRITE ( 14 )  u_av
    ENDIF
    WRITE ( 14 )  'u_m                 ';  WRITE ( 14 )  u_m
    WRITE ( 14 )  'us                  ';  WRITE ( 14 )  us
    WRITE ( 14 )  'usws                ';  WRITE ( 14 )  usws
    IF ( timestep_scheme(1:5) /= 'runge' )  THEN
       WRITE ( 14 )  'usws_m              ';  WRITE ( 14 )  usws_m
    ENDIF
    IF ( ALLOCATED( us_av ) )  THEN
       WRITE ( 14 )  'us_av               ';  WRITE ( 14 )  us_av
    ENDIF
    WRITE ( 14 )  'v                   ';  WRITE ( 14 )  v
    WRITE ( 14 )  'volume_flow_area    ';  WRITE ( 14 )  volume_flow_area
    WRITE ( 14 )  'volume_flow_initial ';  WRITE ( 14 )  volume_flow_initial
    IF ( ALLOCATED( v_av ) )  THEN
       WRITE ( 14 )  'v_av                ';  WRITE ( 14 )  v_av
    ENDIF
    WRITE ( 14 )  'v_m                 ';  WRITE ( 14 )  v_m
    IF ( moisture )  THEN
       WRITE ( 14 )  'vpt                 ';  WRITE ( 14 )  vpt
       IF ( ALLOCATED( vpt_av ) )  THEN
          WRITE ( 14 )  'vpt_av              ';  WRITE ( 14 )  vpt_av
       ENDIF
       IF ( timestep_scheme(1:5) /= 'runge' )  THEN
          WRITE ( 14 )  'vpt_m               ';  WRITE ( 14 )  vpt_m
       ENDIF
    ENDIF
    WRITE ( 14 )  'vsws                ';  WRITE ( 14 )  vsws
    IF ( timestep_scheme(1:5) /= 'runge' )  THEN
       WRITE ( 14 )  'vsws_m              ';  WRITE ( 14 )  vsws_m
    ENDIF
    WRITE ( 14 )  'w                   ';  WRITE ( 14 )  w
    IF ( ALLOCATED( w_av ) )  THEN
       WRITE ( 14 )  'w_av                ';  WRITE ( 14 )  w_av
    ENDIF
    WRITE ( 14 )  'w_m                 ';  WRITE ( 14 )  w_m
    WRITE ( 14 )  'z0                  ';  WRITE ( 14 )  z0

    WRITE ( 14 )  'cross_linecolors    ';  WRITE ( 14 )  cross_linecolors
    WRITE ( 14 )  'cross_linestyles    ';  WRITE ( 14 )  cross_linestyles
    WRITE ( 14 )  'cross_normalized_x  ';  WRITE ( 14 )  cross_normalized_x
    WRITE ( 14 )  'cross_normalized_y  ';  WRITE ( 14 )  cross_normalized_y
    WRITE ( 14 )  'cross_normx_factor  ';  WRITE ( 14 )  cross_normx_factor
    WRITE ( 14 )  'cross_normy_factor  ';  WRITE ( 14 )  cross_normy_factor
    WRITE ( 14 )  'cross_profiles      ';  WRITE ( 14 )  cross_profiles
    WRITE ( 14 )  'cross_profile_n_coun'
                  WRITE ( 14 )  cross_profile_number_count
    WRITE ( 14 )  'cross_profile_number';  WRITE ( 14 )  cross_profile_numbers
    WRITE ( 14 )  'cross_uxmax         ';  WRITE ( 14 )  cross_uxmax
    WRITE ( 14 )  'cross_uxmax_computed';  WRITE ( 14 )  cross_uxmax_computed
    WRITE ( 14 )  'cross_uxmax_normaliz';  WRITE ( 14 )  cross_uxmax_normalized
    WRITE ( 14 )  'cross_uxmax_norm_com'
                  WRITE ( 14 )  cross_uxmax_normalized_computed
    WRITE ( 14 )  'cross_uxmin         ';  WRITE ( 14 )  cross_uxmin
    WRITE ( 14 )  'cross_uxmin_computed';  WRITE ( 14 )  cross_uxmin_computed
    WRITE ( 14 )  'cross_uxmin_normaliz';  WRITE ( 14 )  cross_uxmin_normalized
    WRITE ( 14 )  'cross_uxmin_norm_com'
                  WRITE ( 14 )  cross_uxmin_normalized_computed
    WRITE ( 14 )  'cross_uymax         ';  WRITE ( 14 )  cross_uymax
    WRITE ( 14 )  'cross_uymin         ';  WRITE ( 14 )  cross_uymin
    WRITE ( 14 )  'cross_xtext         ';  WRITE ( 14 )  cross_xtext
    WRITE ( 14 )  'dopr_crossindex     ';  WRITE ( 14 )  dopr_crossindex
    WRITE ( 14 )  'dopr_time_count     ';  WRITE ( 14 )  dopr_time_count
    WRITE ( 14 )  'hom_sum             ';  WRITE ( 14 )  hom_sum
    WRITE ( 14 )  'profile_columns     ';  WRITE ( 14 )  profile_columns
    WRITE ( 14 )  'profile_number      ';  WRITE ( 14 )  profile_number
    WRITE ( 14 )  'profile_rows        ';  WRITE ( 14 )  profile_rows
    IF ( ALLOCATED( spectrum_x ) )  THEN
       WRITE ( 14 )  'spectrum_x          ';  WRITE ( 14 )  spectrum_x
       WRITE ( 14 )  'spectrum_y          ';  WRITE ( 14 )  spectrum_y
    ENDIF

!
!-- Write end label. Unit 14 is closed in the main program.
    WRITE ( 14 )  '*** end ***         '


    CALL cpu_log( log_point(22), 'write_3d_binary', 'stop' )


 END SUBROUTINE write_3d_binary
